from typing import cast

from zyntex import SourceCode
from zyntex.syntax import FunctionDeclaration
from zyntex.bindings import PrimitiveType


class TestSourceCode:

    def test_basic_usage(self):
        code = SourceCode("fn testing() void {}")
        assert len(code.content) == 1
        function_decl: FunctionDeclaration = cast(FunctionDeclaration, code.content[0])
        assert function_decl.name == "testing"
        assert function_decl.public is False
        assert function_decl.body == "{}"

        assert function_decl.return_type.is_type()
        assert not function_decl.return_type.is_const()
        assert function_decl.return_type.absolute_type == PrimitiveType.void



import ctypes
from typing import List

from .enums import ErrorTag, NodeTag, TokenTag


class PyString:
    """A special type for GenericSlice to treat an entire array of bytes as a single py string."""


class TranslationUnit(ctypes.Structure):
    """Represents a parsed translation unit from the Zig AST."""


class GenericSlice(ctypes.Structure):
    """A generic container for FFI slices, representing a pointer and it's length."""

    _fields_ = [
        ("ptr", ctypes.c_void_p),
        ("len", ctypes.c_size_t),
    ]

    def __repr__(self) -> str:
        return f"GenericSlice(len={self.len}, ptr={self.ptr})"

    def to_list(self, ctype: type, encoding="utf-8") -> List:
        """Return the slice as a Python list.

        If `ctype` is `PyString`, the entire underlying byte array is decoded into
        a single Python string and returned as a list with one element.
        For other ctypes, the slice is returned as a list of individual instances.

        May raise a EOFError if the slice is empty."""
        if self.is_empty:
            raise EOFError("Slice is empty.")
        if ctype is PyString:
            buf = bytes((ctypes.c_char * self.len).from_address(self.ptr))
            return [buf.decode(encoding)]

        ptr = ctypes.cast(self.ptr, ctypes.POINTER(ctype))
        return [ptr[i] for i in range(self.len)]

    @property
    def is_empty(self) -> bool:
        """Whether the slice is empty."""
        return self.ptr is None


class ASTNode(ctypes.Structure):
    """Represents a node in the Zig AST."""

    _fields_ = [
        ("index", ctypes.c_int),
        ("tag_index", ctypes.c_int),
        ("main_token", ctypes.c_int),
        ("lhs", ctypes.c_int),
        ("rhs", ctypes.c_int),
    ]

    def __repr__(self) -> str:
        return (
            f"ASTNode("
            f"tag={self.tag}, "
            f"index={self.index}, "
            f"main_token={self.main_token}, "
            f"lhs={self.lhs}, "
            f"rhs={self.rhs})"
        )

    @property
    def tag(self) -> NodeTag:
        """The node tag as a `NodeTag` enum."""
        return NodeTag(self.tag_index)


class ASTToken(ctypes.Structure):
    """Represents a token in the Zig AST."""

    _fields_ = [
        ("tag_index", ctypes.c_int),
        ("start", ctypes.c_int),
    ]

    def __repr__(self) -> str:
        return f"ASTToken(tag={self.tag}, start={self.start})"

    @property
    def tag(self) -> TokenTag:
        """The token tag as a `TokenTag` enum."""
        return TokenTag(self.tag_index)


class ErrorReport(ctypes.Structure):
    """Represents a parsing error."""

    _fields_ = [
        ("tag_index", ctypes.c_int),
        ("is_note", ctypes.c_bool),
        ("token_is_prev", ctypes.c_bool),
        ("token_index", ctypes.c_int),
    ]

    def __repr__(self) -> str:
        return (
            f"ErrorData("
            f"tag={self.tag}, "
            f"is_note={self.is_note}, "
            f"token_is_prev={self.token_is_prev}, "
            f"token_index={self.token_index})"
        )

    @property
    def tag(self) -> ErrorTag:
        """The error tag as an `ErrorTag` enum."""
        return ErrorTag(self.tag_index)


class NodeParam(ctypes.Structure):
    """Represents a node parameter."""

    _fields_ = [
        ("name", GenericSlice),
        ("type", ASTNode)
    ]

    def __repr__(self) -> str:
        return f"NodeParam(name={self.name}, type={self.type})"



from enum import Enum


# pylint: disable=invalid-name


class NodeTag(Enum):
    """
    Bindings to the Zig `std.zig.Ast.Node.Tag` enum.
    Ref: https://github.com/ziglang/zig/blob/0.14.1/lib/std/zig/Ast.zig#L3038

    These enum values correspond to the numeric values returned by Zig parsing API.
    """
    ROOT = 0
    USING_NAMESPACE = 1
    TEST_DECL = 2
    GLOBAL_VAR_DECL = 3
    LOCAL_VAR_DECL = 4
    SIMPLE_VAR_DECL = 5
    ALIGNED_VAR_DECL = 6
    ERRDEFER = 7
    DEFER = 8
    CATCH = 9
    FIELD_ACCESS = 10
    UNWRAP_OPTIONAL = 11
    EQUAL_EQUAL = 12
    BANG_EQUAL = 13
    LESS_THAN = 14
    GREATER_THAN = 15
    LESS_OR_EQUAL = 16
    GREATER_OR_EQUAL = 17
    ASSIGN_MUL = 18
    ASSIGN_DIV = 19
    ASSIGN_MOD = 20
    ASSIGN_ADD = 21
    ASSIGN_SUB = 22
    ASSIGN_SHL = 23
    ASSIGN_SHL_SAT = 24
    ASSIGN_SHR = 25
    ASSIGN_BIT_AND = 26
    ASSIGN_BIT_XOR = 27
    ASSIGN_BIT_OR = 28
    ASSIGN_MUL_WRAP = 29
    ASSIGN_ADD_WRAP = 30
    ASSIGN_SUB_WRAP = 31
    ASSIGN_MUL_SAT = 32
    ASSIGN_ADD_SAT = 33
    ASSIGN_SUB_SAT = 34
    ASSIGN = 35
    ASSIGN_DESTRUCTURE = 36
    MERGE_ERROR_SETS = 37
    MUL = 38
    DIV = 39
    MOD = 40
    ARRAY_MULT = 41
    MUL_WRAP = 42
    MUL_SAT = 43
    ADD = 44
    SUB = 45
    ARRAY_CAT = 46
    ADD_WRAP = 47
    SUB_WRAP = 48
    ADD_SAT = 49
    SUB_SAT = 50
    SHL = 51
    SHL_SAT = 52
    SHR = 53
    BIT_AND = 54
    BIT_XOR = 55
    BIT_OR = 56
    ORELSE = 57
    BOOL_AND = 58
    BOOL_OR = 59
    BOOL_NOT = 60
    NEGATION = 61
    BIT_NOT = 62
    NEGATION_WRAP = 63
    ADDRESS_OF = 64
    TRY = 65
    AWAIT = 66
    OPTIONAL_TYPE = 67
    ARRAY_TYPE = 68
    ARRAY_TYPE_SENTINEL = 69
    PTR_TYPE_ALIGNED = 70
    PTR_TYPE_SENTINEL = 71
    PTR_TYPE = 72
    PTR_TYPE_BIT_RANGE = 73
    SLICE_OPEN = 74
    SLICE = 75
    SLICE_SENTINEL = 76
    DEREF = 77
    ARRAY_ACCESS = 78
    ARRAY_INIT_ONE = 79
    ARRAY_INIT_ONE_COMMA = 80
    ARRAY_INIT_DOT_TWO = 81
    ARRAY_INIT_DOT_TWO_COMMA = 82
    ARRAY_INIT_DOT = 83
    ARRAY_INIT_DOT_COMMA = 84
    ARRAY_INIT = 85
    ARRAY_INIT_COMMA = 86
    STRUCT_INIT_ONE = 87
    STRUCT_INIT_ONE_COMMA = 88
    STRUCT_INIT_DOT_TWO = 89
    STRUCT_INIT_DOT_TWO_COMMA = 90
    STRUCT_INIT_DOT = 91
    STRUCT_INIT_DOT_COMMA = 92
    STRUCT_INIT = 93
    STRUCT_INIT_COMMA = 94
    CALL_ONE = 95
    CALL_ONE_COMMA = 96
    ASYNC_CALL_ONE = 97
    ASYNC_CALL_ONE_COMMA = 98
    CALL = 99
    CALL_COMMA = 100
    ASYNC_CALL = 101
    ASYNC_CALL_COMMA = 102
    SWITCH = 103
    SWITCH_COMMA = 104
    SWITCH_CASE_ONE = 105
    SWITCH_CASE_INLINE_ONE = 106
    SWITCH_CASE = 107
    SWITCH_CASE_INLINE = 108
    SWITCH_RANGE = 109
    WHILE_SIMPLE = 110
    WHILE_CONT = 111
    WHILE = 112
    FOR_SIMPLE = 113
    FOR = 114
    FOR_RANGE = 115
    IF_SIMPLE = 116
    IF = 117
    SUSPEND = 118
    RESUME = 119
    CONTINUE = 120
    BREAK = 121
    RETURN = 122
    FN_PROTO_SIMPLE = 123
    FN_PROTO_MULTI = 124
    FN_PROTO_ONE = 125
    FN_PROTO = 126
    FN_DECL = 127
    ANYFRAME_TYPE = 128
    ANYFRAME_LITERAL = 129
    CHAR_LITERAL = 130
    NUMBER_LITERAL = 131
    UNREACHABLE_LITERAL = 132
    IDENTIFIER = 133
    ENUM_LITERAL = 134
    STRING_LITERAL = 135
    MULTILINE_STRING_LITERAL = 136
    GROUPED_EXPRESSION = 137
    BUILTIN_CALL_TWO = 138
    BUILTIN_CALL_TWO_COMMA = 139
    BUILTIN_CALL = 140
    BUILTIN_CALL_COMMA = 141
    ERROR_SET_DECL = 142
    CONTAINER_DECL = 143
    CONTAINER_DECL_TRAILING = 144
    CONTAINER_DECL_TWO = 145
    CONTAINER_DECL_TWO_TRAILING = 146
    CONTAINER_DECL_ARG = 147
    CONTAINER_DECL_ARG_TRAILING = 148
    TAGGED_UNION = 149
    TAGGED_UNION_TRAILING = 150
    TAGGED_UNION_TWO = 151
    TAGGED_UNION_TWO_TRAILING = 152
    TAGGED_UNION_ENUM_TAG = 153
    TAGGED_UNION_ENUM_TAG_TRAILING = 154
    CONTAINER_FIED_INIT = 155
    CONTAINER_FIELD_ALIGN = 156
    CONTAINER_FIELD = 157
    COMPTIME = 158
    NOSUSPEND = 159
    BLOCK_TWO = 160
    BLOCK_TWO_SEMICOLON = 161
    BLOCK = 162
    BLOCK_SEMICOLON = 163
    ASM_SIMPLE = 164
    ASM = 165
    ASM_OUTPUT = 166
    ASM_INPUT = 167
    ERROR_VALUE = 168
    ERROR_UNION = 169


class TokenTag(Enum):
    """
    Bindings to the Zig `std.zig.tokenizer.Token.Tag` enum.
    Ref: https://github.com/ziglang/zig/blob/0.14.1/lib/std/zig/tokenizer.zig#L68
    """
    INVALID = 0
    INVALID_PERIODASTERISKS = 1
    IDENTIFIER = 2
    STRING_LITERAL = 3
    MULTILINE_STRING_LITERAL_LINE = 4
    CHAR_LITERAL = 5
    EOF = 6
    BUILTIN = 7
    BANG = 8
    PIPE = 9
    PIPE_PIPE = 10
    PIPE_EQUAL = 11
    EQUAL = 12
    EQUAL_EQUAL = 13
    EQUAL_ANGLE_BRACKET_RIGHT = 14
    BANG_EQUAL = 15
    L_PAREN = 16
    R_PAREN = 17
    SEMICOLON = 18
    PERCENT = 19
    PERCENT_EQUAL = 20
    L_BRACE = 21
    R_BRACE = 22
    L_BRACKET = 23
    R_BRACKET = 24
    PERIOD = 25
    PERIOD_ASTERISK = 26
    ELLIPSIS2 = 27
    ELLIPSIS3 = 28
    CARET = 29
    CARET_EQUAL = 30
    PLUS = 31
    PLUS_PLUS = 32
    PLUS_EQUAL = 33
    PLUS_PERCENT = 34
    PLUS_PERCENT_EQUAL = 35
    PLUS_PIPE = 36
    PLUS_PIPE_EQUAL = 37
    MINUS = 38
    MINUS_EQUAL = 39
    MINUS_PERCENT = 40
    MINUS_PERCENT_EQUAL = 41
    MINUS_PIPE = 42
    MINUS_PIPE_EQUAL = 43
    ASTERISK = 44
    ASTERISK_EQUAL = 45
    ASTERISK_ASTERISK = 46
    ASTERISK_PERCENT = 47
    ASTERISK_PERCENT_EQUAL = 48
    ASTERISK_PIPE = 49
    ASTERISK_PIPE_EQUAL = 50
    ARROW = 51
    COLON = 52
    SLASH = 53
    SLASH_EQUAL = 54
    COMMA = 55
    AMPERSAND = 56
    AMPERSAND_EQUAL = 57
    QUESTION_MARK = 58
    ANGLE_BRACKET_LEFT = 59
    ANGLE_BRACKET_LEFT_EQUAL = 60
    ANGLE_BRACKET_ANGLE_BRACKET_LEFT = 61
    ANGLE_BRACKET_ANGLE_BRACKET_LEFT_EQUAL = 62
    ANGLE_BRACKET_ANGLE_BRACKET_LEFT_PIPE = 63
    ANGLE_BRACKET_ANGLE_BRACKET_LEFT_PIPE_EQUAL = 64
    ANGLE_BRACKET_RIGHT = 65
    ANGLE_BRACKET_RIGHT_EQUAL = 66
    ANGLE_BRACKET_ANGLE_BRACKET_RIGHT = 67
    ANGLE_BRACKET_ANGLE_BRACKET_RIGHT_EQUAL = 68
    TILDE = 69
    NUMBER_LITERAL = 70
    DOC_COMMENT = 71
    CONTAINER_DOC_COMMENT = 72
    KEYWORD_ADDRSPACE = 73
    KEYWORD_ALIGN = 74
    KEYWORD_ALLOWZERO = 75
    KEYWORD_AND = 76
    KEYWORD_ANYFRAME = 77
    KEYWORD_ANYTYPE = 78
    KEYWORD_ASM = 79
    KEYWORD_ASYNC = 80
    KEYWORD_AWAIT = 81
    KEYWORD_BREAK = 82
    KEYWORD_CALLCONV = 83
    KEYWORD_CATCH = 84
    KEYWORD_COMPTIME = 85
    KEYWORD_CONST = 86
    KEYWORD_CONTINUE = 87
    KEYWORD_DEFER = 88
    KEYWORD_ELSE = 89
    KEYWORD_ENUM = 90
    KEYWORD_ERRDEFER = 91
    KEYWORD_ERROR = 92
    KEYWORD_EXPORT = 93
    KEYWORD_EXTERN = 94
    KEYWORD_FN = 95
    KEYWORD_FOR = 96
    KEYWORD_IF = 97
    KEYWORD_INLINE = 98
    KEYWORD_NOALIAS = 99
    KEYWORD_NOINLINE = 100
    KEYWORD_NOSUSPEND = 101
    KEYWORD_OPAQUE = 102
    KEYWORD_OR = 103
    KEYWORD_ORELSE = 104
    KEYWORD_PACKED = 105
    KEYWORD_PUB = 106
    KEYWORD_RESUME = 107
    KEYWORD_RETURN = 108
    KEYWORD_LINKSECTION = 109
    KEYWORD_STRUCT = 110
    KEYWORD_SUSPEND = 111
    KEYWORD_SWITCH = 112
    KEYWORD_TEST = 113
    KEYWORD_THREADLOCAL = 114
    KEYWORD_TRY = 115
    KEYWORD_UNION = 116
    KEYWORD_UNREACHABLE = 117
    KEYWORD_USINGNAMESPACE = 118
    KEYWORD_VAR = 119
    KEYWORD_VOLATILE = 120
    KEYWORD_WHILE = 121


class ErrorTag(Enum):
    """
    Bindings to the Zig `std.zig.Ast.Error.Tag` enum.
    Ref: https://github.com/ziglang/zig/blob/0.14.1/lib/std/zig/Ast.zig#L2950

    These enum values correspond to the error tag identifiers returned by the Zig parser.
    Used to indicate specific kinds of syntax or semantic errors encountered during parsing.
    """
    ASTERISK_AFTER_PTR_DEREF = 0
    CHAINED_COMPARISON_OPERATORS = 1
    DECL_BETWEEN_FIELDS = 2
    EXPECTED_BLOCK = 3
    EXPECTED_BLOCK_OR_ASSIGNMENT = 4
    EXPECTED_BLOCK_OR_EXPR = 5
    EXPECTED_BLOCK_OR_FIELD = 6
    EXPECTED_CONTAINER_MEMBERS = 7
    EXPECTED_EXPR = 8
    EXPECTED_EXPR_OR_ASSIGNMENT = 9
    EXPECTED_EXPR_OR_VAR_DECL = 10
    EXPECTED_FN = 11
    EXPECTED_INLINABLE = 12
    EXPECTED_LABELABLE = 13
    EXPECTED_PARAM_LIST = 14
    EXPECTED_PREFIX_EXPR = 15
    EXPECTED_PRIMARY_TYPE_EXPR = 16
    EXPECTED_PUB_ITEM = 17
    EXPECTED_RETURN_TYPE = 18
    EXPECTED_SEMI_OR_ELSE = 19
    EXPECTED_SEMI_OR_LBRACE = 20
    EXPECTED_STATEMENT = 21
    EXPECTED_SUFFIX_OP = 22
    EXPECTED_TYPE_EXPR = 23
    EXPECTED_VAR_DECL = 24
    EXPECTED_VAR_DECL_OR_FN = 25
    EXPECTED_LOOP_PAYLOAD = 26
    EXPECTED_CONTAINER = 27
    EXTERN_FN_BODY = 28
    EXTRA_ADDRSPACE_QUALIFIER = 29
    EXTRA_ALIGN_QUALIFIER = 30
    EXTRA_ALLOWZERO_QUALIFIER = 31
    EXTRA_CONST_QUALIFIER = 32
    EXTRA_VOLATILE_QUALIFIER = 33
    PTR_MOD_ON_ARRAY_CHILD_TYPE = 34
    INVALID_BIT_RANGE = 35
    SAME_LINE_DOC_COMMENT = 36
    UNATTACHED_DOC_COMMENT = 37
    TEST_DOC_COMMENT = 38
    COMPTIME_DOC_COMMENT = 39
    VARARGS_NONFINAL = 40
    EXPECTED_CONTINUE_EXPR = 41
    EXPECTED_SEMI_AFTER_DECL = 42
    EXPECTED_SEMI_AFTER_STMT = 43
    EXPECTED_COMMA_AFTER_FIELD = 44
    EXPECTED_COMMA_AFTER_ARG = 45
    EXPECTED_COMMA_AFTER_PARAM = 46
    EXPECTED_COMMA_AFTER_INITIALIZER = 47
    EXPECTED_COMMA_AFTER_SWITCH_PRONG = 48
    EXPECTED_COMMA_AFTER_FOR_OPERAND = 49
    EXPECTED_COMMA_AFTER_CAPTURE = 50
    EXPECTED_INITIALIZER = 51
    MISMATCHED_BINARY_OP_WHITESPACE = 52
    INVALID_AMPERSAND_AMPERSAND = 53
    C_STYLE_CONTAINER = 54
    EXPECTED_VAR_CONST = 55
    WRONG_EQUAL_VAR_DECL = 56
    VAR_CONST_DECL = 57
    EXTRA_FOR_CAPTURE = 58
    FOR_INPUT_NOT_CAPTURED = 59
    ZIG_STYLE_CONTAINER = 60
    PREVIOUS_FIELD = 61
    NEXT_FIELD = 62
    EXPECTED_TOKEN = 63
    INVALID_BYTE = 64


class PrimitiveType(Enum):
    """
    Bindings for Zig primitive types.
    Ref: https://ziglang.org/documentation/0.14.1/#Primitive-Types
    """
    i8 = "i8"
    i16 = "i16"
    i32 = "i32"
    i64 = "i64"
    i128 = "i128"

    u8 = "u8"
    u16 = "u16"
    u32 = "u32"
    u64 = "u64"
    u128 = "u128"

    isize = "isize"
    usize = "usize"

    c_char = "c_char"
    c_short = "c_short"
    c_ushort = "c_ushort"

    c_int = "c_int"
    c_uint = "c_uint"
    c_long = "c_long"
    c_ulong = "c_ulong"

    c_longlong = "c_longlong"
    c_ulonglong = "c_ulonglong"
    c_longdouble = "c_longdouble"

    f16 = "f16"
    f32 = "f32"
    f64 = "f64"
    f80 = "f80"
    f128 = "f128"

    bool = "bool"
    anyopaque = "anyopaque"
    void = "void"
    noreturn = "noreturn"
    type = "type"
    anyerror = "anyerror"
    comptime_int = "comptime_int"
    comptime_float = "comptime_float"



from typing import Optional

from ..bindings import PyASTNode, NodeTag
from .node_element import INodeElement
from .type_node import TypeNode


class VariableDeclaration(INodeElement):
    """Represents a Zig variable declaration."""

    __slots__ = ("_node",)

    def __init__(self, node: PyASTNode) -> None:
        self._node = node
        assert (self.is_node_valid(node)), "Provided node is not a variable declaration."

    def __repr__(self) -> str:
        return f"VariableDeclaration(name={self.name})"

    @staticmethod
    def is_node_valid(node: PyASTNode) -> bool:
        return node.tag in (
            NodeTag.SIMPLE_VAR_DECL,
            NodeTag.LOCAL_VAR_DECL,
            NodeTag.GLOBAL_VAR_DECL,
            NodeTag.ALIGNED_VAR_DECL
        )

    @property
    def name(self) -> str:
        return self._node.spelling

    @property
    def type_hint(self) -> Optional[TypeNode]:
        """The explicit Zig type annotation for the variable, if present."""
        node_type = self._node.type
        if node_type:
            return TypeNode(node_type)

    @property
    def alignment(self) -> Optional[str]:
        """The explicit Zig alignment for the variable, if present."""
        if self._node.tag == NodeTag.ALIGNED_VAR_DECL:
            return self._node.lhs_source
        if self._node.tag in (NodeTag.LOCAL_VAR_DECL, NodeTag.GLOBAL_VAR_DECL):
            return self._node.align

    @property
    def value(self) -> Optional[str]:
        """The raw value of the variable. None if the variable is declared with extern."""
        if not self.extern:
            return self._node.rhs_source

    @property
    def public(self) -> bool:
        """Whether the variable is marked as pub."""
        return self._node.is_public()

    @property
    def const(self) -> bool:
        """Whether the variable is declared with const."""
        return self._node.is_const()

    @property
    def extern(self) -> bool:
        """True, if the variable is declared with extern."""
        return self._node.is_extern()

    @property
    def export(self) -> bool:
        """True, if the variable is declared with export."""
        return self._node.is_export()



from typing import cast
from pathlib import Path

from zyntex import SourceFile
from zyntex.syntax import FunctionDeclaration
from zyntex.bindings import PrimitiveType


class TestSourceFile:

    def test_basic_usage(self):
        file = SourceFile(file_path=str(self.path_to_test_sources / "basic.zig"))
        assert file.path == str(self.path_to_test_sources / "basic.zig")
        assert len(file.content) == 1
        function_decl: FunctionDeclaration = cast(FunctionDeclaration, file.content[0])
        assert function_decl.name == "testing"
        assert function_decl.public is False
        assert function_decl.body == "{}"

        assert function_decl.return_type.is_type()
        assert not function_decl.return_type.is_const()
        assert function_decl.return_type.absolute_type == PrimitiveType.void

    @property
    def path_to_test_sources(self) -> Path:
        return Path(__file__).resolve().parent / "test_sources"



import ctypes
import os

from dataclasses import dataclass
from typing import Optional, Tuple

from .structures import ASTNode, GenericSlice, NodeParam
from .translation_unit import TranslationUnitPtr

_lib_instance: Optional[ctypes.CDLL] = None


@dataclass
class FunctionSignature:
    name: str
    return_type: Optional[type]
    arg_types: Tuple[type, ...]


lib_functions = [
    FunctionSignature("createTranslationUnit", TranslationUnitPtr, (ctypes.c_char_p,)),
    FunctionSignature("createTranslationUnitFromSource", TranslationUnitPtr, (ctypes.c_char_p,)),
    FunctionSignature("getTranslationUnitNodesCount", ctypes.c_size_t, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitNodes", GenericSlice, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitRootNodes", GenericSlice, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitNodeFromIndex", ASTNode,
                      (TranslationUnitPtr, ctypes.c_uint32)),
    FunctionSignature("getTranslationUnitTokensCount", ctypes.c_size_t, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitTokens", GenericSlice, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitErrorsCount", ctypes.c_size_t, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitErrors", GenericSlice, (TranslationUnitPtr,)),
    FunctionSignature("getTranslationUnitSource", GenericSlice, (TranslationUnitPtr,)),
    FunctionSignature("freeTranslationUnit", None, (TranslationUnitPtr,)),

    FunctionSignature("getNodeSpelling", GenericSlice, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("getNodeSource", GenericSlice, (TranslationUnitPtr, ctypes.c_uint32)),
    FunctionSignature("getNodeType", ASTNode, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("getNodeAlign", GenericSlice, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("getNodeBody", GenericSlice, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("getNodeParamsCount", ctypes.c_size_t, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("getNodeParams", ctypes.c_size_t,
                      (TranslationUnitPtr, ASTNode, ctypes.POINTER(NodeParam), ctypes.c_size_t)),
    FunctionSignature("isNodeExtern", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodeExport", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodePublic", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodeConst", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodeContainer", ctypes.c_bool, (ASTNode,)),
    FunctionSignature("isNodeStruct", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodeUnion", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodeOpaque", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
    FunctionSignature("isNodeEnum", ctypes.c_bool, (TranslationUnitPtr, ASTNode)),
]


def get_lib_ext() -> str:
    from sys import platform  # pylint: disable=import-outside-toplevel

    exts = {"win32": "dll", "darwin": "dylib", "linux": "so"}
    if ext := exts.get(platform):
        return ext
    raise ValueError(f"Invalid platform: '{platform}'.")


def load_native_library(lib_dir: Optional[str] = None) -> ctypes.CDLL:
    lib_ext: str = get_lib_ext()
    path = lib_dir or os.path.join(os.path.dirname(os.path.realpath(__file__)), "native")
    ctypes_lib = ctypes.CDLL(f"{path}/clib.{lib_ext}")

    for func in lib_functions:
        lib_func = getattr(ctypes_lib, func.name)
        setattr(lib_func, "argtypes", func.arg_types)
        setattr(lib_func, "restype", func.return_type)
    return ctypes_lib


def get_native_library(lib_dir: Optional[str] = None) -> ctypes.CDLL:
    global _lib_instance  # pylint: disable=global-statement
    if _lib_instance is None:
        _lib_instance = load_native_library(lib_dir)
    return _lib_instance



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional, cast, Union

from ..bindings import PyASTNode, NodeTag, PrimitiveType
from .node_element import INodeElement


class TypeNode(INodeElement):
    """A High-level wrapper for Zig Types.

    Provides methods to check if a type is an array, optional,
    pointer, or constant, and to access related subtypes.
    Distinguishes between primitive and custom (user-defined) types."""

    @dataclass
    class CustomType:
        """Represents a non-primitive (user-defined) type name."""
        value: str

    __slots__ = ("_node",)

    def __init__(self, node: PyASTNode) -> None:
        self._node = node
        assert self.is_node_valid(node), "Provided node is not a type."

    def __repr__(self) -> str:
        return (f"NodeType("
                f"spelling={self._node.spelling}, "
                f"is_array={self.is_array()}, "
                f"is_optional={self.is_optional()}, "
                f"is_ptr={self.is_ptr()}, "
                f"is_type={self.is_type()})")

    @staticmethod
    def is_node_valid(node: PyASTNode) -> bool:
        return node.tag in (
            NodeTag.OPTIONAL_TYPE,
            NodeTag.ARRAY_TYPE,
            NodeTag.IDENTIFIER,
            NodeTag.PTR_TYPE_ALIGNED
        )

    def is_array(self) -> bool:
        return self._node.tag == NodeTag.ARRAY_TYPE

    def is_optional(self) -> bool:
        return self._node.tag == NodeTag.OPTIONAL_TYPE

    def is_ptr(self) -> bool:
        return self._node.tag == NodeTag.PTR_TYPE_ALIGNED

    def is_type(self) -> bool:
        return self._node.tag == NodeTag.IDENTIFIER

    def is_const(self) -> bool:
        return self._node.is_const()

    @property
    def spelling(self) -> str:
        """The spelling of the current node."""
        return self._node.spelling

    @property
    def array_length(self) -> Optional[str]:
        """Declared length of the array.

        The value is returned as a string because it may represent
        either a numeric literal or a symbolic name (variable/expression)."""
        if self.is_array():
            return self._node.lhs_source

    @property
    def array_type(self) -> Optional[TypeNode]:
        """Type node of the array, or None if not an array."""
        if self.is_array():
            return TypeNode(self._node.rhs_node)

    @property
    def optional_type(self) -> Optional[TypeNode]:
        """The inner type of optional, or None if not optional."""
        if self.is_optional():
            return TypeNode(self._node.lhs_node)

    @property
    def ptr_type(self) -> Optional[TypeNode]:
        """The pointed-to type, or None if not a pointer."""
        if self.is_ptr():
            return TypeNode(self._node.rhs_node)

    @property
    def type(self) -> Optional[PrimitiveType | CustomType]:
        """Immediate type as PrimitiveType or CustomType, if identifier."""
        if self.is_type():
            value = self._node.spelling
            try:
                return PrimitiveType(value)
            except ValueError:
                return self.CustomType(value)

    @property
    def absolute_type(self) -> Union[TypeNode.CustomType, PrimitiveType]:
        """Resolve the absolute base type.

        Walks through optional/array/pointer wrappers to locate the underlying
        identifier. Returns `PrimitiveType` for known primitives or `CustomType`
        for user-defined names. Raises `NotImplementedError` for unsupported nodes."""
        current_node: TypeNode = self
        while True:
            if current_node.is_type():
                return cast(Union[TypeNode.CustomType, PrimitiveType], current_node.type)
            if current_node.is_optional():
                current_node = cast(TypeNode, current_node.optional_type)
            elif current_node.is_array():
                current_node = cast(TypeNode, current_node.array_type)
            elif current_node.is_ptr():
                current_node = cast(TypeNode, current_node.ptr_type)
            else:
                raise NotImplementedError(f"Type node ({current_node}) is not supported.")



from dataclasses import dataclass

from .source_file import SourceFile
from .source_module import SourceModule
from .source_code import SourceCode


@dataclass
class Version:
    major: int
    minor: int
    patch: int

    def __str__(self) -> str:
        return f"{self.major}.{self.minor}.{self.patch}"


__all__ = (
    "SourceFile",
    "SourceModule",
    "SourceCode",
    "__version__"
)

__version__ = Version(0, 0, 2)



from .structures import (
    TranslationUnit, GenericSlice,
    ASTNode, ASTToken, ErrorReport, NodeParam, PyString
)
from .translation_unit import PyTranslationUnit, TranslationUnitPtr
from .enums import NodeTag, TokenTag, ErrorTag, PrimitiveType
from .native import load_native_library, get_native_library
from .ast_node import PyASTNode

__all__ = (
    "TranslationUnit",
    "GenericSlice",
    "ASTNode",
    "ASTToken",
    "ErrorReport",
    "NodeParam",
    "PyString",
    "PyTranslationUnit",
    "TranslationUnit",
    "NodeTag",
    "TokenTag",
    "ErrorTag",
    "PrimitiveType",
    "PyASTNode",
    "load_native_library",
    "get_native_library"
)



from typing import Optional, List
from dataclasses import dataclass

from ..bindings import PyASTNode, NodeTag, PyString
from .node_element import INodeElement
from .type_node import TypeNode


class FunctionDeclaration(INodeElement):
    """Represents a Zig function declaration."""

    @dataclass
    class FunctionParam:
        name: str
        type: TypeNode

    __slots__ = ("_node",)

    def __init__(self, node: PyASTNode) -> None:
        self._node = node
        assert (self.is_node_valid(node)), "Provided node is not a function declaration."

    def __repr__(self) -> str:
        return f"FunctionDeclaration(name={self.name})"

    @staticmethod
    def is_node_valid(node: PyASTNode) -> bool:
        if node.tag == NodeTag.FN_DECL:
            return True
        return node.is_extern and node.tag in (
            NodeTag.FN_PROTO,
            NodeTag.FN_PROTO_ONE,
            NodeTag.FN_PROTO_SIMPLE,
            NodeTag.FN_PROTO_MULTI
        )

    @property
    def name(self) -> str:
        return self._node.spelling

    @property
    def body(self) -> Optional[str]:
        """Raw body of the function declaration. None if the function is declared with extern."""
        if self._node.tag == NodeTag.FN_DECL:
            return self._node.rhs_source

    @property
    def return_type(self) -> TypeNode:
        return_type: Optional[PyASTNode] = self._node.type
        assert return_type is not None, "The function doesn't have a return type."
        return TypeNode(return_type)

    @property
    def public(self) -> bool:
        """Whether the function is marked as pub."""
        return self._node.is_public()

    @property
    def extern(self) -> bool:
        """True, if the function is declared with extern."""
        return self._node.is_extern()

    @property
    def export(self) -> bool:
        """True, if the function is declared with export."""
        return self._node.is_export()

    @property
    def params(self) -> List[FunctionParam]:
        """List of function parameters."""
        result = []
        for param_data in self._node.params:
            result.append(
                FunctionDeclaration.FunctionParam(
                    name=param_data.name.to_list(PyString)[0],
                    type=TypeNode(PyASTNode(self._node.parent, param_data.type)),
                )
            )
        return result



from __future__ import annotations
from typing import TYPE_CHECKING, Optional, List

from .structures import ASTNode, NodeParam, PyString

if TYPE_CHECKING:
    from .translation_unit import PyTranslationUnit
    from .enums import NodeTag


class PyASTNode:
    """Represents a high-level Python wrapper for a single `ASTNode`.
    Provides convenient accessors for commonly used node information."""

    __slots__ = ("_parent", "_node", "_lib")

    def __init__(self, parent: PyTranslationUnit, node: ASTNode) -> None:
        self._parent = parent
        self._node = node
        self._lib = parent.lib

    def __repr__(self) -> str:
        return f"PyASTNode(tag={self.tag})"

    def is_public(self) -> bool:
        """True if the node is marked as pub."""
        return self._lib.isNodePublic(self._parent.ptr, self._node)

    def is_extern(self) -> bool:
        """True if the node is marked as extern."""
        return self._lib.isNodeExtern(self._parent.ptr, self._node)

    def is_export(self) -> bool:
        """True if the node is marked as export."""
        return self._lib.isNodeExport(self._parent.ptr, self._node)

    def is_container(self) -> bool:
        """True if the node is a container (Enum, Struct, Union, or Opaque)."""
        return self._lib.isNodeContainer(self._node)

    def is_const(self) -> bool:
        """Whether the node is marked as const."""
        return self._lib.isNodeConst(self._parent.ptr, self._node)

    def is_struct(self) -> bool:
        """Whether the node points to a struct."""
        return self._lib.isNodeStruct(self._parent.ptr, self._node)

    def is_union(self) -> bool:
        """Whether the node points to a union."""
        return self._lib.isNodeUnion(self._parent.ptr, self._node)

    def is_opaque(self) -> bool:
        """Whether the node points to an opaque."""
        return self._lib.isNodeOpaque(self._parent.ptr, self._node)

    def is_enum(self) -> bool:
        """Whether the node points to an enum."""
        return self._lib.isNodeEnum(self._parent.ptr, self._node)

    @property
    def tag(self) -> NodeTag:
        """The node's tag."""
        return self._node.tag

    @property
    def spelling(self) -> str:
        """The node's spelling as a decoded UTF-8 string."""
        return self._lib.getNodeSpelling(
            self._parent.ptr, self._node
        ).to_list(PyString)[0]

    @property
    def rhs_source(self) -> str:
        """The source text for the node's right-hand side (RHS)."""
        return self._lib.getNodeSource(
            self._parent.ptr, self.rhs
        ).to_list(PyString)[0]

    @property
    def rhs(self) -> int:
        """The index of the node right-hand side (RHS)."""
        return self._node.rhs

    @property
    def rhs_node(self) -> PyASTNode:
        """The node's right-hand side (RHS)."""
        ast_node = self._lib.getTranslationUnitNodeFromIndex(self._parent.ptr, self.rhs)
        return PyASTNode(parent=self.parent, node=ast_node)

    @property
    def lhs_source(self) -> str:
        """The source text for the node's left-hand side (LHS)."""
        return self._lib.getNodeSource(
            self._parent.ptr, self.lhs
        ).to_list(PyString)[0]

    @property
    def lhs(self) -> int:
        """The index of the node left-hand side (LHS)."""
        return self._node.lhs

    @property
    def lhs_node(self) -> PyASTNode:
        """The node's left-hand side (LHS)."""
        ast_node = self._lib.getTranslationUnitNodeFromIndex(self._parent.ptr, self.lhs)
        return PyASTNode(parent=self.parent, node=ast_node)

    @property
    def type(self) -> Optional[PyASTNode]:
        """The type node assigned to Node. For functions, it's the return type.
        For variable declarations, it specifies the type hint."""
        node_type = self._lib.getNodeType(self._parent.ptr, self._node)
        if node_type.index != self._node.index:
            return PyASTNode(self.parent, node_type)

    @property
    def body(self) -> Optional[str]:
        """Node's raw body."""
        body_slice = self._lib.getNodeBody(self._parent.ptr, self._node)
        if not body_slice.is_empty:
            return body_slice.to_list(PyString)[0]

    @property
    def params(self) -> List[NodeParam]:
        """A list of node's parameters. For functions, these are arguments"""
        result = []
        params_count = self._lib.getNodeParamsCount(self._parent.ptr, self._node)
        if params_count == 0:
            return result
        buffer = (NodeParam * params_count)()
        filled = self._lib.getNodeParams(self._parent.ptr, self._node, buffer, params_count)
        for i in range(filled):
            result.append(buffer[i])
        return result

    @property
    def align(self) -> Optional[str]:
        """The align value for the node."""
        align_slice = self._lib.getNodeAlign(self._parent.ptr, self._node)
        if not align_slice.is_empty:
            return align_slice.to_list(PyString)[0]

    @property
    def released(self) -> bool:
        """Indicates whether the ASTNode has been released."""
        return self._parent.released

    @property
    def parent(self) -> PyTranslationUnit:
        """The parent of this ASTNode."""
        return self._parent



from __future__ import annotations

from ctypes import POINTER, CDLL, c_uint32
from typing import Optional, Any

from .structures import ErrorReport, TranslationUnit, ASTNode, ASTToken, PyString
from .ast_node import PyASTNode

TranslationUnitPtr = POINTER(TranslationUnit)


class PyTranslationUnit:
    """Represents a parsed translation unit for a single source file.

    Manages parsing, memory management, and provides convenient access
    to the underlying translation unit's resources at a higher level.
    """
    __slots__ = ("_tu_ptr", "_lib", "_path", "_released")

    def __init__(self, lib: CDLL, tu_ptr: Any, path: Optional[str] = None) -> None:
        self._tu_ptr = tu_ptr
        self._lib = lib
        self._path = path or "null"
        self._released = False

        if not self._tu_ptr:
            raise RuntimeError(
                f"Failed to parse translation unit from path: '{self._path}'. "
                f"The file may be missing, unreadable, or contain unrecoverable syntax errors."
            )

    @classmethod
    def from_path(cls, lib: CDLL, path: str) -> PyTranslationUnit:
        translation_unit_ptr = lib.createTranslationUnit(path.encode())
        return cls(lib=lib, tu_ptr=translation_unit_ptr, path=path)

    @classmethod
    def from_source(cls, lib: CDLL, source: str) -> PyTranslationUnit:
        translation_unit_ptr = lib.createTranslationUnitFromSource(source.encode())
        return cls(lib=lib, tu_ptr=translation_unit_ptr)

    def __del__(self) -> None:
        # Ensure resources are released when the instance is garbage collected.
        self.release()

    def __repr__(self) -> str:
        return f"PyTranslationUnit(path={self._path}, released={self._released})"

    def nodes_count(self) -> int:
        """GetS the total number of AST nodes within this translation unit."""
        return self._lib.getTranslationUnitNodesCount(self._tu_ptr)

    def tokens_count(self) -> int:
        """Retrieves the count of tokens parsed in the translation unit."""
        return self._lib.getTranslationUnitTokensCount(self._tu_ptr)

    def errors_count(self) -> int:
        """Retrieves the count of errors parsed in the translation unit."""
        return self._lib.getTranslationUnitErrorsCount(self._tu_ptr)

    def nodes(self) -> list[PyASTNode]:
        """Returns a list of AST nodes parsed in the translation unit."""
        nodes = self._lib.getTranslationUnitNodes(self._tu_ptr).to_list(ASTNode)
        return list(map(lambda n: PyASTNode(self, n), nodes))

    def root_nodes(self) -> list[PyASTNode]:
        """Returns the root AST nodes parsed in the translation unit."""
        root_nodes = []
        index_list = self._lib.getTranslationUnitRootNodes(self._tu_ptr).to_list(c_uint32)
        for index in index_list:
            node = self._lib.getTranslationUnitNodeFromIndex(self._tu_ptr, index)
            root_nodes.append(PyASTNode(self, node))
        return root_nodes

    def tokens(self) -> list[ASTToken]:
        """Returns a list of AST tokens parsed in the translation unit."""
        return self._lib.getTranslationUnitTokens(self._tu_ptr).to_list(ASTToken)

    def errors(self) -> list[ErrorReport]:
        """Returns a list of ErrorReport instances for all errors encountered during parsing.
        Parsing continues despite errors, so this list may contain multiple reports."""
        return self._lib.getTranslationUnitErrors(self._tu_ptr).to_list(ErrorReport)

    def release(self) -> None:
        """Manually release TranslationUnit memory.
        Once completed, unit resources will be no longer available."""
        if not self._released:
            self._released = True
            if self._tu_ptr:
                self._lib.freeTranslationUnit(self._tu_ptr)

    @property
    def source(self) -> str:
        """Fetches the full source code as a decoded UTF-8 string."""
        return self._lib.getTranslationUnitSource(self._tu_ptr).to_list(PyString)[0]

    @property
    def path(self) -> str:
        """Returns the original file path used for parsing."""
        return self._path

    @property
    def released(self) -> bool:
        """Whether the translation unit has been released."""
        return self._released

    @property
    def ptr(self) -> Any:
        """Gets the low-level pointer to the translation unit."""
        return self._tu_ptr

    @property
    def lib(self) -> CDLL:
        """Gets the low-level ctypes.CDLL handle for the native parser library."""
        return self._lib



from __future__ import annotations
from typing import List, Type, Tuple

from .syntax import INodeElement, FunctionDeclaration, VariableDeclaration, TestDeclaration
from .bindings import PyTranslationUnit, ErrorReport, get_native_library


class SourceCode:
    """Represents a parsed source code.

    Parses the given source text into a translation unit and provides
    access to recognized top-level node elements."""

    __slots__ = ("_unit", "_source")

    def __init__(self, source: str) -> None:
        self._unit = PyTranslationUnit.from_source(lib=get_native_library(), source=source)
        self._source = source

    def __repr__(self) -> str:
        return f"SourceFile(size={len(self._source)})"

    @property
    def content(self) -> List[INodeElement]:
        """A list of top-level elements parsed from the source string."""
        result = []
        for node in self._unit.root_nodes():
            for node_type in self.types:
                if node_type.is_node_valid(node):
                    result.append(node_type(node))
        return result

    @property
    def types(self) -> Tuple[Type[INodeElement], ...]:
        """Supported top-level node element types."""
        return FunctionDeclaration, VariableDeclaration, TestDeclaration

    @property
    def errors(self) -> List[ErrorReport]:
        """A list of error reports that occurred during parsing this file."""
        return self._unit.errors()



from abc import ABC, abstractmethod


class INodeElement(ABC):
    """Base class for all Zig AST node wrappers."""

    @abstractmethod
    def __init__(self, *args, **kwargs):
        raise NotImplementedError

    @staticmethod
    @abstractmethod
    def is_node_valid(node) -> bool:
        raise NotImplementedError



from ..bindings import PyASTNode, NodeTag
from .node_element import INodeElement


class TestDeclaration(INodeElement):
    """Represents a Zig test declaration."""

    __slots__ = ("_node",)

    def __init__(self, node: PyASTNode) -> None:
        self._node = node
        assert (self.is_node_valid(node)), "Provided node is not a test declaration."

    def __repr__(self) -> str:
        return f"TestDeclaration(name={self.name})"

    @staticmethod
    def is_node_valid(node: PyASTNode) -> bool:
        return node.tag == NodeTag.TEST_DECL

    @property
    def name(self) -> str:
        return self._node.spelling

    @property
    def body(self) -> str:
        """Raw body of the test."""
        return self._node.rhs_source



from typing import cast

import subprocess
import sys

from setuptools import Command, setup, find_packages
from setuptools.command.build_py import build_py as _build_py
from pathlib import Path

from zyntex import __version__


class build_py(_build_py):
    def run(self):
        subprocess.check_call([sys.executable, "build_libs.py", "--release-mode"])
        super().run()


here = Path(__file__).parent
long_description = (here / "README.md").read_text(encoding="utf-8") \
    if (here / "README.md").exists() else ""

setup(
    name="zyntex",
    version=str(__version__),
    description="A high-level Python package to parse Zig code.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="xXenvy",
    url="https://github.com/xXenvy/zyra",
    project_urls={
        "Source": "https://github.com/xXenvy/zyra",
        "Tracker": "https://github.com/xXenvy/zyra/issues",
    },
    license="MIT",
    packages=find_packages(exclude=("tests", "zig")),
    include_package_data=True,
    package_data={"zyntex": ["bindings/native/*"]},
    python_requires=">=3.9",
    install_requires=[],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Software Development :: Compilers",
        "Topic :: Software Development :: Libraries",
    ],
    keywords="zig parser codegen",
    zip_safe=False,
    cmdclass={"build_py": cast(type[Command], build_py)},
)



from .function_declaration import FunctionDeclaration
from .variable_declaration import VariableDeclaration
from .test_declaration import TestDeclaration
from .node_element import INodeElement
from .type_node import TypeNode

__all__ = (
    "INodeElement",
    "FunctionDeclaration",
    "VariableDeclaration",
    "TestDeclaration",
    "TypeNode"
)



from typing import List
from os import walk, path

from .source_file import SourceFile


class SourceModule:
    """A SourceModule, which allows you to parse multiple .zig files.
    From the provided path, all .zig files will be extracted."""

    __slots__ = ("_dir_path",)

    def __init__(self, dir_path: str) -> None:
        self._dir_path = dir_path

    def __repr__(self) -> str:
        return f"SourceModule(dir={self._dir_path})"

    @property
    def files(self) -> List[SourceFile]:
        """A list of SourceFile for all `.zig` files in the directory."""
        sources = []
        for root, _, files in walk(self._dir_path):
            for filename in files:
                if filename.endswith(".zig"):
                    full_path = path.join(root, filename)
                    sources.append(SourceFile(full_path))
        return sources

    @property
    def dir_path(self) -> str:
        """Provided path to the directory."""
        return self._dir_path



from zyntex import SourceModule
from pathlib import Path


class TestSourceModule:

    def test_basic_usage(self):
        module = SourceModule(dir_path=str(self.path_to_test_sources / "test_module"))
        files = module.files
        assert len(files) == 2
        assert files[0].path == str(self.path_to_test_sources / "test_module" / "src.zig")
        assert files[1].path == str(self.path_to_test_sources / "test_module" / "inside" / "src2.zig")

    @property
    def path_to_test_sources(self) -> Path:
        return Path(__file__).resolve().parent / "test_sources"
